import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { dispatchEvent } from '@s-ui/js/lib/events';
var DETECTION_DELAY = 5000;
var activationHandlers = {};
var optimizelyPromise;
var wasExperimentsListenerAdded = false;

/**
 * Waits until func returns a value and calls callback with it
 * @param  {Function}   truthyFn       Function to execute to check value
 * @param  {Function} callback       Callback called passing value
 * @param  {Number}   [delay=100]    Delay allowed to wait for value
 * @param  {Number}   [interval=100] Interval time for checks
 */
var waitUntil = function waitUntil(truthyFn, callback, delay, interval) {
  if (delay === void 0) {
    delay = 100;
  }
  if (interval === void 0) {
    interval = 100;
  }
  var intervalId = setInterval(function () {
    var value = truthyFn();
    if (value || delay <= 0) {
      clearInterval(intervalId);
      callback(value);
    }
    delay -= interval;
  }, interval);
};
var getOptimizely = function getOptimizely() {
  return window && window.optimizely && window.optimizely.get && window.optimizely;
};

/**
 * Register handler to optimizely ONCE
 * @param {Object} sdk OptimizelySdk
 */
var registerExprimentsEvents = function registerExprimentsEvents(sdk) {
  if (sdk && !wasExperimentsListenerAdded) {
    sdk.push({
      type: 'addListener',
      filter: {
        type: 'lifecycle',
        name: 'campaignDecided'
      },
      handler: experimentsActivationHandler
    });
    wasExperimentsListenerAdded = true;
  }
};

/**
 * Handler for Optimizely events
 * @param {Object} event
 */
var experimentsActivationHandler = function experimentsActivationHandler(event) {
  var _event$data$decision = event.data.decision,
    experimentId = _event$data$decision.experimentId,
    variationId = _event$data$decision.variationId;
  dispatchActivated(experimentId, variationId);
};

/**
 * Executed all handlers added to an experiment with variationId as param
 * @param {String} experimentId
 * @param {String} variationId
 */
var dispatchActivated = function dispatchActivated(experimentId, variationId) {
  var handlers = activationHandlers[experimentId] || [];
  handlers.forEach(function (handler) {
    return handler(variationId);
  });
};
var OptimizelyXExperiments = /*#__PURE__*/function () {
  function OptimizelyXExperiments() {}
  /**
   * Get OptimizelyX SDK instance
   * @return Promise<Object>
   */
  OptimizelyXExperiments.getSDK =
  /*#__PURE__*/
  function () {
    var _getSDK = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              optimizelyPromise = optimizelyPromise || new Promise(function (resolve) {
                waitUntil(getOptimizely, function (sdk) {
                  registerExprimentsEvents(sdk);
                  resolve(sdk);
                }, DETECTION_DELAY);
              });
              return _context.abrupt("return", optimizelyPromise);
            case 2:
            case "end":
              return _context.stop();
          }
        }
      }, _callee);
    }));
    function getSDK() {
      return _getSDK.apply(this, arguments);
    }
    return getSDK;
  }()
  /**
   * Get info of an experiment
   * @param {String} experimentId
   * @return Promise<Object>
   * @example
   * {
   *  id: '100',
   *  isActive: true,
   *  isInExperimentHoldback: true,
   *  variation: {
   *      id: '101',
   *      name: 'Variation #1'
   *  }
   * }
   */
  ;
  OptimizelyXExperiments.getInfo =
  /*#__PURE__*/
  function () {
    var _getInfo = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(experimentId) {
      var sdk;
      return _regeneratorRuntime.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return this.getSDK();
            case 2:
              sdk = _context2.sent;
              return _context2.abrupt("return", sdk && sdk.get && sdk.get('state').getExperimentStates()[experimentId]);
            case 4:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));
    function getInfo(_x) {
      return _getInfo.apply(this, arguments);
    }
    return getInfo;
  }()
  /**
   * Get if an experiment is activated for current user
   * @param {String} experimentId
   * @return Promise<Boolean>
   */
  ;
  OptimizelyXExperiments.isActivated =
  /*#__PURE__*/
  function () {
    var _isActivated = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(experimentId) {
      var info;
      return _regeneratorRuntime.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return this.getInfo(experimentId);
            case 2:
              _context3.t0 = _context3.sent;
              if (_context3.t0) {
                _context3.next = 5;
                break;
              }
              _context3.t0 = {};
            case 5:
              info = _context3.t0;
              return _context3.abrupt("return", info.isActive && !info.isInExperimentHoldback || false);
            case 7:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3, this);
    }));
    function isActivated(_x2) {
      return _isActivated.apply(this, arguments);
    }
    return isActivated;
  }()
  /**
   * Add function to call when experiment gets activated.
   * If experiment is already activated, the handler is called right away.
   * @param {String} experimentId
   * @param {Function} handler
   */
  ;
  OptimizelyXExperiments.addActivationListener =
  /*#__PURE__*/
  function () {
    var _addActivationListener = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee4(experimentId, handler) {
      var variationId;
      return _regeneratorRuntime.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              activationHandlers[experimentId] = activationHandlers[experimentId] || [];
              activationHandlers[experimentId].push(handler);
              _context4.next = 4;
              return this.isActivated(experimentId);
            case 4:
              if (!_context4.sent) {
                _context4.next = 10;
                break;
              }
              _context4.next = 7;
              return this.getVariation(experimentId);
            case 7:
              variationId = _context4.sent;
              handler(variationId);
              variationId && dispatchEvent({
                eventName: 'OptimizelyActivated',
                detail: {
                  experimentId: experimentId,
                  variationId: variationId
                }
              });
            case 10:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4, this);
    }));
    function addActivationListener(_x3, _x4) {
      return _addActivationListener.apply(this, arguments);
    }
    return addActivationListener;
  }()
  /**
   * Remove function to call when experiment gets activated.
   * @param {String} experimentId
   * @param {Function} removedHandler
   */
  ;
  OptimizelyXExperiments.removeActivationListener =
  /*#__PURE__*/
  function () {
    var _removeActivationListener = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee5(experimentId, removedHandler) {
      var handlers;
      return _regeneratorRuntime.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              handlers = activationHandlers[experimentId];
              if (handlers) {
                _context5.next = 3;
                break;
              }
              return _context5.abrupt("return");
            case 3:
              activationHandlers[experimentId] = handlers.filter(function (handler) {
                return handler !== removedHandler;
              });
            case 4:
            case "end":
              return _context5.stop();
          }
        }
      }, _callee5);
    }));
    function removeActivationListener(_x5, _x6) {
      return _removeActivationListener.apply(this, arguments);
    }
    return removeActivationListener;
  }()
  /**
   * Get variation to display for current user
   * @param {String} experimentId
   * @return Promise<String|Number>
   */
  ;
  OptimizelyXExperiments.getVariation =
  /*#__PURE__*/
  function () {
    var _getVariation = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee6(experimentId) {
      var info;
      return _regeneratorRuntime.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              _context6.next = 2;
              return this.getInfo(experimentId);
            case 2:
              _context6.t0 = _context6.sent;
              if (_context6.t0) {
                _context6.next = 5;
                break;
              }
              _context6.t0 = {};
            case 5:
              info = _context6.t0;
              return _context6.abrupt("return", info.variation.id || null);
            case 7:
            case "end":
              return _context6.stop();
          }
        }
      }, _callee6, this);
    }));
    function getVariation(_x7) {
      return _getVariation.apply(this, arguments);
    }
    return getVariation;
  }();
  return OptimizelyXExperiments;
}();
export default OptimizelyXExperiments;